package ru.amse.kanzheleva.moviemaker.player;


import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ru.amse.kanzheleva.moviemaker.movie.IMovie;
import ru.amse.kanzheleva.moviemaker.ui.frame.actions.PlayerOpenAction;
import ru.amse.kanzheleva.moviemaker.xml.MovieLoader;

public class MovieBox extends JFrame{
	private static final long serialVersionUID = 1L;
	private JSlider myTimeSlider;
	private IMovie myMovie;
	private MoviePlayer myMoviePlayer;
	private SpeedDialog mySpeedDialog;
	private JButton myButton;
	private MovieLoader myLoader;
	
	public MovieBox(IMovie movie, String title) {
		super(title);
		setSize(new Dimension(700, 700));
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		myLoader = new MovieLoader();
		myMovie = movie;
		
		myMoviePlayer = new MoviePlayer(this, myMovie, 600, 600, 1, 1, 4);
		add(myMoviePlayer);
		
		mySpeedDialog = new SpeedDialog(this, "Frame per second", myMovie.getFPS());
		
		JMenuBar menuBar = new JMenuBar();
    	setJMenuBar(menuBar);
    	createFileMenu(menuBar);
    	createSpeedMenu(menuBar);
    	
    	myButton = createStartPauseButton();
    	createTimeSlider();
    	
    	addContent();
	}
	
	private void addContent() {
		JPanel buttonTimeSliderContentPane = new JPanel();
    	buttonTimeSliderContentPane.setLayout(new BoxLayout(buttonTimeSliderContentPane, 
    			BoxLayout.X_AXIS));
    	buttonTimeSliderContentPane.add(myButton);
    	buttonTimeSliderContentPane.add(myTimeSlider);
    	
    	JPanel fieldButtonContentPane = new JPanel();
    	fieldButtonContentPane.setLayout(new BoxLayout(fieldButtonContentPane,
    			BoxLayout.Y_AXIS));
    	fieldButtonContentPane.add(myMoviePlayer);
    	fieldButtonContentPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    	fieldButtonContentPane.add(buttonTimeSliderContentPane);
    	
    	setContentPane(fieldButtonContentPane);		
	}
	
	private final void createFileMenu(JMenuBar menuBar) {
		JMenu fileMenu = new JMenu("File");
    	JMenuItem openItem = new JMenuItem("Open");
    	openItem.setAction(new PlayerOpenAction(MovieBox.this));
    	openItem.setText("Open");
    	JMenuItem closeItem = new JMenuItem("Close");
    	closeItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}  		
    	});
    	fileMenu.add(openItem);
    	fileMenu.addSeparator();
    	fileMenu.add(closeItem);
    	menuBar.add(fileMenu);
	}
	
	private final void createSpeedMenu(JMenuBar menuBar) {
		JMenu speedMenu = new JMenu("Speed");
    	JMenuItem item = new JMenuItem("Frames per second");
    	item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				myMoviePlayer.setNewDelay(mySpeedDialog.openDialog());
			}  		
    	});
    	speedMenu.add(item);
      	menuBar.add(speedMenu);
	}
	

	public void ableMove(boolean able) {
		myButton.setEnabled(able);
	}
	
	private final JButton createStartPauseButton() {
		final JButton button = new JButton("start");
    	button.setAlignmentX(Component.LEFT_ALIGNMENT);
     	button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				 if (myMoviePlayer.isFrozen()) {
		                myMoviePlayer.setIsFrozen(false);	
		                button.setText("pause");
		                myMoviePlayer.startAnimation();
		            } else {
		                myMoviePlayer.setIsFrozen(true);
                        button.setText("start");
		                myMoviePlayer.stopAnimation();
		            }
			}  		
    	});
    	return button;
	}
	
	private final void createTimeSlider() { 
		myTimeSlider = new JSlider(JSlider.HORIZONTAL, 0, 
        		myMovie.getSize() - 1, 0);
        myTimeSlider.setMajorTickSpacing(1);
        myTimeSlider.setPaintTicks(true);
        myTimeSlider.setPaintLabels(true);
        myTimeSlider.addChangeListener(new TimeSliderListener());        
	}
	
	 private class TimeSliderListener implements ChangeListener {
		 public void stateChanged(ChangeEvent e) {
			 JSlider sourse = (JSlider)e.getSource();
			 if (!sourse.getValueIsAdjusting()) {
				 int timePassed = (int)sourse.getValue();
				 myTimeSlider.setValue(timePassed);
				 myMoviePlayer.setNewFrame(timePassed);
			 }
		 }   		
	 }
	 
	public void moveTimeSlider() {
		 myTimeSlider.setValue(myTimeSlider.getValue() + 1);
	 }
	 
	 public void resetTimeSliderAndButton() {
		 myTimeSlider.setValue(0);
		 myButton.setText("start");
	 }
	 
	 public void setNewMovie(File file) {
		 myMovie = myLoader.read(file.getAbsolutePath());
		 myMoviePlayer.setNewMovie(myMovie);
		 mySpeedDialog.setFPS(myMovie.getFPS());
		 myTimeSlider.setMaximum(myMovie.getSize() - 1);
	 }
	

}
