package ru.amse.kanzheleva.moviemaker.ui.frame;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BoxLayout;
import javax.swing.JColorChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.xml.stream.XMLStreamException;

import ru.amse.kanzheleva.moviemaker.movie.Frame;
import ru.amse.kanzheleva.moviemaker.movie.IFrame;
import ru.amse.kanzheleva.moviemaker.movie.IMovie;
import ru.amse.kanzheleva.moviemaker.movie.figures.Ellipse;
import ru.amse.kanzheleva.moviemaker.movie.figures.PolyLine;
import ru.amse.kanzheleva.moviemaker.movie.figures.Polygon;
import ru.amse.kanzheleva.moviemaker.movie.figures.Rectangle;
import ru.amse.kanzheleva.moviemaker.movie.figures.RoundRectangle;
import ru.amse.kanzheleva.moviemaker.ui.frame.actions.EditOpenAction;
import ru.amse.kanzheleva.moviemaker.ui.frame.actions.EditSaveAction;
import ru.amse.kanzheleva.moviemaker.ui.frameeditor.FrameEditor;
import ru.amse.kanzheleva.moviemaker.ui.frameeditor.IChangaListener;
import ru.amse.kanzheleva.moviemaker.ui.previewpane.IChooseListener;
import ru.amse.kanzheleva.moviemaker.ui.previewpane.PreviewPane;
import ru.amse.kanzheleva.moviemaker.xml.MovieLoader;
import ru.amse.kanzheleva.moviemaker.xml.MovieSaver;
import ru.amse.kanzheleva.moviemaker.xml.WrongDataException;

public class FrameMaker {
	private final FrameEditor myDrawField;
	private PreviewPane myPreviewPane;
	private JMenuBar myMenuBar;
	private IMovie myMovie;
	private AddPanel myAddPanel;
	private JScrollPane myScroller;
		
	private MovieSaver myMovieSaver = new MovieSaver();
	private MovieLoader myMovieLoader = new MovieLoader();
	
	int width = 600; 
	int height = 600;
	
	public FrameMaker(final IMovie movie) {
		myMovie = movie;
				
		JFrame frame = new JFrame("Frame viewer");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
						
		myDrawField = new FrameEditor(myMovie.getFrame(0), width, height, 1, 1);
		fillDrawField();
		
		myAddPanel = new AddPanel();
		myAddPanel.setEnable(false);
		addListeners(myAddPanel);		
		
		fillMenu();
		frame.setJMenuBar(myMenuBar);
						
		myPreviewPane = new PreviewPane(movie, 0.2, width, height);
		myPreviewPane.setMaximumSize(new Dimension(2 * width, (int) (height * 0.2 + 10)));
		myPreviewPane.addListener(new IChooseListener() {
			public void frameChosen(int index) {
				myDrawField.setFrame(myMovie.getFrame(index));
			}					
		});		
		
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		contentPane.add(myAddPanel);
		contentPane.add(myScroller);
		contentPane.add(myPreviewPane);
		
		frame.getContentPane().add(contentPane);
		frame.pack();
		frame.setVisible(true);
		
	}	
	
	private final void fillDrawField() {
		myScroller = new JScrollPane(myDrawField);
		myScroller.setPreferredSize(new Dimension(500,  500));
		myDrawField.setPreferredSize(new Dimension(width, height));
		myDrawField.addChangeListener(new IChangaListener() {
			public void frameChange() {
				myPreviewPane.repaint();
			}

			public void figureSelected(boolean isSelected) {
				myAddPanel.setEnable(isSelected);
			}			
		});
	}
	
	private void addListeners(AddPanel addPanel) {
		addPanel.addEllipseListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				myDrawField.addFigure(new Ellipse(), 0);
			}			
		});
		
		addPanel.addRectangleListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				myDrawField.addFigure(new Rectangle(), 0);
			}			
		});
		
		addPanel.addRoundRectangleListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				myDrawField.addFigure(new RoundRectangle(), 0);
			}			
		});
		
		addPanel.addPolyLineListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int n;
				String N = JOptionPane.showInputDialog("count of vertex");
				if (N == null) {
					return;
				} else {
					if (isRightData(N.toString())) {
						n = Integer.valueOf(N).intValue();
						if (n < 2) {
							JOptionPane.showMessageDialog(null,
								    "wrong value", null, JOptionPane.ERROR_MESSAGE);
							return;
						}
					} else {
						JOptionPane.showMessageDialog(null,
							    "only digit not more 999", null, JOptionPane.ERROR_MESSAGE);
						return;
					}
				}
				myDrawField.addFigure(new PolyLine(), n);
			}
		});
		
		addPanel.addPolygonListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int n;
				String N = JOptionPane.showInputDialog("count of vertex");
				if (N == null) {
					return;
				} else {
					if (isRightData(N.toString())) {
						n = Integer.valueOf(N).intValue();
						if (n <= 2) {
							JOptionPane.showMessageDialog(null,
								    "wrong value", null, JOptionPane.ERROR_MESSAGE);
							return;
						}
					} else {
						JOptionPane.showMessageDialog(null,
							    "only digit not more 999", null, JOptionPane.ERROR_MESSAGE);
						return;
					}
				}				
				myDrawField.addFigure(new Polygon(), n);
			}
		});
		
		addPanel.addFillColorListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Color color = JColorChooser.showDialog(myDrawField, "Color Chooser", Color.red);
				if (color != null) {
					myDrawField.setFillColor(color);
					myPreviewPane.repaint();
				}
			}
			
		});
		
		addPanel.addLineColorListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Color color = JColorChooser.showDialog(myDrawField, "Color Chooser", Color.red);
				if (color != null) {
					myDrawField.setLineColor(color);
					myPreviewPane.repaint();
				}
			}
			
		});
		
		addPanel.addDeleteListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				myDrawField.deleteFigure();
			}			
		});
	}
	
	private boolean isRightData(String data) {
		char[] symbols = data.toCharArray();
		if (symbols.length > 3) {
			return false;
		}  
		for (int i = 0; i < symbols.length; i++) {
			if (!Character.isDigit(symbols[i])) {
				return false;
			}
		}
		return true;			
	}
	
	private void fillMenu() {
		myMenuBar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");
		
		JMenuItem openItem = new JMenuItem("Open");
		EditOpenAction openAction = new EditOpenAction(this);	
		openItem.setAction(openAction);
		openItem.setText("Open");
		fileMenu.add(openItem);
		
		
		JMenuItem saveItem = new JMenuItem("Save");
		EditSaveAction saveAction = new EditSaveAction(this);
		saveItem.setAction(saveAction);
		saveItem.setText("Save");
		fileMenu.add(saveItem);
		
		myMenuBar.add(fileMenu);
		
		JMenu newMenu = new JMenu("New");
		JMenuItem newFrameItem = new JMenuItem("New frame");
		newFrameItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addNewFrame();
			}			
		});
		newMenu.add(newFrameItem);
		myMenuBar.add(newMenu);
	}
 
	private void addNewFrame() {
		IFrame frame = new Frame();
		myMovie.addFrame(frame);
		myDrawField.setFrame(frame);
		myPreviewPane.addNewFrame(frame);
	}
	
	public void openNewMovie(File file) {
		try {
			myMovie = myMovieLoader.read(file.getAbsolutePath());
			myPreviewPane.setNewMovie(myMovie);
			myDrawField.setFrame(myMovie.getFrame(0));
		} catch (WrongDataException e) {
			throw e;
		}
	}
	
	
	public void saveMovie(File file) throws WrongDataException, XMLStreamException {
		myMovieSaver.write(file.getAbsolutePath(), myMovie);
	}
	
	public void openMovie(File file) {
		myMovie = myMovieLoader.read(file.getName());	
	}
}
