package ru.amse.kanzheleva.moviemaker.xml;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import ru.amse.kanzheleva.moviemaker.movie.Frame;
import ru.amse.kanzheleva.moviemaker.movie.IFrame;
import ru.amse.kanzheleva.moviemaker.movie.IGraphicalObject;
import ru.amse.kanzheleva.moviemaker.movie.IMovie;
import ru.amse.kanzheleva.moviemaker.movie.Movie;
import ru.amse.kanzheleva.moviemaker.movie.figures.Ellipse;
import ru.amse.kanzheleva.moviemaker.movie.figures.PolyLine;
import ru.amse.kanzheleva.moviemaker.movie.figures.Polygon;
import ru.amse.kanzheleva.moviemaker.movie.figures.Rectangle;
import ru.amse.kanzheleva.moviemaker.movie.figures.RoundRectangle;

public class MovieLoader {
	public IMovie read(String name) throws WrongDataException {
		InputStream inputStream = null;
		IMovie movie = new Movie();
		try {
			inputStream = new FileInputStream(name);
			SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
			parser.parse(inputStream, new Handler(movie));
			inputStream.close();
		} catch (WrongDataException e) {
			throw e;
		} catch (IOException e) {
			throw new WrongDataException("IO problem");
		} catch (ParserConfigurationException e) {
			throw new WrongDataException("IO problem");
		} catch (SAXException e) {
			throw new WrongDataException("XML problem");
		} catch (NumberFormatException e) {
			throw new WrongDataException("Wrong data");			
		}finally {
			if (inputStream != null) {
				try {
					inputStream.close();
				} catch (IOException e) {
					throw new WrongDataException("IO problem");
				}
			}
		} 
		return movie;
	}
	
	private static class Handler extends DefaultHandler {
		private IMovie myMovie;
		private IState myState;
		private IFrame myFrame;
		private IGraphicalObject myFigure;
		private IState beforeMovie = new BeforeMovie();
		private IState beforeFrame = new BeforeFrame();
		private IState beforeFigure = new BeforeFigure();
		private IState readPolygonAndPolyLine = new ReadPolygoneAndPolyLine();
		
		public Handler(IMovie movie) {
			super();
			myMovie = movie;
			myState = beforeMovie;
		}
		
		public void startElement(String uri, String localName, String elementName, Attributes attributes) {
			myState = myState.startElement(uri, localName, elementName, attributes);
		}
		
		public void endElement(String uri, String localName, String elementName) {
			if (MovieConstants.FRAME.equals(elementName)) {
				myState = beforeFrame;
				myMovie.addFrame(myFrame);
			}
		}

		
		private interface IState {
			public IState startElement(String uri, String localName, String elementName, Attributes attributes);
		}
		
		private class BeforeMovie implements IState {
			public IState startElement(String uri, String localName,
					String elementName, Attributes attributes) {
				if (MovieConstants.MOVIE.equals(elementName) && (attributes.getLength() == 1)) {
					myMovie.setFPS(Integer.valueOf(attributes.getValue(MovieConstants.FPS)));
					return beforeFrame;
				} else {
					throw new WrongDataException("Wrong Data. 'movie' expacted.");
				}
			}
		}
		
		private class BeforeFrame implements IState {
			public IState startElement(String uri, String localName,
					String elementName, Attributes attributes) {
				if (MovieConstants.FRAME.equals(elementName)) {
					myFrame = new Frame();
					return beforeFigure;
				} else {
					throw new WrongDataException("Wrong Data. 'frame' expacted.");
				}
			}
		}
		
		private class BeforeFigure implements IState {
			public IState startElement(String uri, String localName,
					String elementName, Attributes attributes) {
				myFigure = null;
			    if (MovieConstants.ELLIPSE.equals(elementName)) {
					myFigure = new Ellipse();
				} else if (MovieConstants.RECTANGLE.equals(elementName)) {
					myFigure = new Rectangle();
				} else if (MovieConstants.ROUNDRECTANGLE.equals(elementName)) {
					myFigure = new RoundRectangle();
				} else if (MovieConstants.POLYGON.equals(elementName)) {
					myFigure = new Polygon();
					GraphicalObjectReader.read(attributes, myFigure);
					return readPolygonAndPolyLine;
				} else if (MovieConstants.POLYLINE.equals(elementName)) {
					myFigure = new PolyLine();
					GraphicalObjectReader.read(attributes, myFigure);
					return readPolygonAndPolyLine;
				} else {
					throw new RuntimeException("Wrong Data. figure name expacted.");
					
				}
				GraphicalObjectReader.read(attributes, myFigure);
				myFigure.visit(new FigureReader(), attributes);
				myFrame.addFigure(myFigure);
				return beforeFigure;
			}
		}
		
		
		private class ReadPolygoneAndPolyLine implements IState {
			public IState startElement(String uri, String localName,
					String elementName, Attributes attributes) {
				if (MovieConstants.POINTS.equals(elementName)) {
					myFigure.visit(new FigureReader(), attributes);
					myFrame.addFigure(myFigure);
					return beforeFigure;
				} else {
					throw new WrongDataException("Wrong Data. 'points' expacted.");
				}
			}
		}
	}	
}
